//
// Copyright 1996, Thomas Andrews (thomaso@best.com)
// Free for non-commercial use
//
#include <stdlib.h>
#include <strstream.h>
#include <fstream.h>
#include <string.h>
#include "Deal.H"

/* 
 *
 *  The ChooseTable is just a fast choose function.
 *  Rather gross, actually.  Once created, it acts
 *  like a function, so that we can call Choose(n,k).
 *
 *  The object just states the limits to which we expect
 *  to call it.
 */
ChooseTable Choose(53,13);

/*
 *  The template class, "tDeal", can be used with other
 *  sizes.  Used templates instead of parameters for
 *      faster computations - will be optimizer-friendly with
 *      good compiler
 *
 */
typedef class tDeal<13,4> Deal;

void source_file(const char *filename) {
  ifstream sourced(filename);
  char buffer[1000];
  while (sourced.getline(buffer,1000)) {
	cout << buffer << endl;
  }
}

void html_headers()
{
  source_file("decode.htmlh");
}

void html_tail()
{
  source_file("decode.htmlt");
}

void html_form(char *arguments, int scramflag) {
  char *ptr;

  for (ptr=arguments; *ptr; ptr++) {
    if ((*ptr)=='+') {
      *ptr = ' ';
    }
  }
	
  cout << "<p> <FORM ACTION=\"index.cgi\">" << endl;
  cout << "Enter a page number or space-seperated list of page numbers:<br>" << endl;
  cout << "<INPUT TYPE=\"text\" NAME=\"isindex\" VALUE=\"" << arguments <<
	"\" SIZE=70 MAXLENGTH=1000></INPUT><br>" << endl;
  cout << "<CENTER>";
  cout << "<INPUT TYPE=\"checkbox\" NAME=\"scramble\" VALUE=\"yes\" " <<
	(scramflag ? "checked" : "")  << ">Scramble</INPUT>\n";
  cout << "<INPUT TYPE=\"submit\" VALUE=\"Decode\"></INPUT>" << endl;
  cout << "</CENTER>" << endl;
  cout << "</FORM>" << endl;

   return ;
}

void replace_cgi_characters(char *string)
{
	char *s=string, *t=string;
	while (*s!=0) {
		if (*s=='%') {
			*t++ = ' ';
			s += 3;
		} else {
			*t++ = *s++;
		}
	}
	*t=0;
}

void weirdo(const char *name)
{
    char *hostname=getenv("REMOTE_HOST");
    if (hostname!=0 && 0==strcmp(hostname,name)) {
	cout << "<h1> Query for the user at " << hostname << "</h1>" << endl;
	source_file("overuse.html");
    }
}

main (int argc, char **argv) {
  int i;
  char *arguments=getenv("QUERY_STRING");
  char *ptr;
  int scramflag=0;
  Integer scramB, scramA; // Parameters to use for scrambling

  html_headers();

  //weirdo("kiatouvu.crim.ca");
  weirdo("thomaso.vip.best.com");

  if (arguments==0 || *arguments==0) {
    html_form("",1);
    html_tail();
    return 0;
  }

  if (0==strncmp("isindex=",arguments,8)) {
	arguments+=8;
  }

  if (ptr=strchr(arguments,'&')) {
	*ptr++=0;
        if (strcmp(ptr,"scramble=yes")==0) {
                // Basically, the scrambling is done via arithmetic
		// mod D, where D is the maximum number of deals
                // We take the input, Index, and do:
		//          trueIndex = A*Index + B (mod D)
                // for a fixed A and B.  Then we look up
		// the deal corresponding to the trueIndex.
		// The only requirement on our A and B is that
		// A has to be relatively prime to D.  This
		// ensures that the map has an inverse.
		//
		// I chose for A the largest power of 11 which was
		// less than D.  11 is the smallest prime which
		// does not divide D, so this A is sure to be
		// relatively prime to D.  Making A large will
		// ensure neighboring hands will be significantly
		// dissimilar. The main fuction of B is to ensure
		// that hand zero isn't a freak.  Therefore, B was
		// set by me typing until I was tired... :-)
		scramA=atoI("13109994191499930367061460371");
		scramB=atoI("34563463456363563565356345634");
		//scramA=atoI("3876269050118516845397872321"); old scrambler
		scramflag=1;
	}
  }

  replace_cgi_characters(arguments);

  istrstream argparser(arguments);

  //  This takes advantage of the happy fact that multiple arguments
  //  are passed with "+" as seperators, which stream >> Integer
  //  handles transparently.
  Integer index;
  const Integer max=atoI("53644737765488792839237439999");
  const Integer modulo=atoI("53644737765488792839237440000");
  while (argparser) { 
    argparser >> index;
    if ((index < 0) || (index > max)) {
      cout << "<h2>Error: " << index << " is out of range</h2>" << endl;
      cout << "Index should be a number from 0 to 53644737765488792839237439999" << endl;
      continue;
    }
    cout << "<h2>In the " << (scramflag ? "Scrambled" : "True") <<
	" Book, on page " << index << ":</h2><p>" << endl;
    if (scramflag) {
        index *= scramA;
	index += scramB;
        index %= modulo;
    }
    Deal d(index);
    cout << "<pre>" << endl;
    d.writelong(cout) << endl;
    cout << "</pre><hr>" << endl;
  }

  html_form(arguments,scramflag);
  html_tail();
  return 0;

}
