//
// Copyright 1996, Thomas Andrews (thomaso@best.com)
// Free for non-commercial use
//
#include "IncrSeq.H"

// An "IncrSequence" is really just a fixed-sized set of
// Integers, listed in increasing order.
//
// It is not meant to be used as a container class - it is
// really just a temporary class used for indexing bridge
// deals.

IncrSequence::IncrSequence(int count, const  int *arr) 
	: _count(count), _arr(new int[count])
{
	int ind;
	for (ind=0; ind<count; ind++) _arr[ind]=arr[ind];
}

//
// "squashedIndex" finds a unique integer index for
// an n-set, using something called the "squashed ordering"
//
// See: Anderson, Combinatorics on Finite Sets, pp 112-119.
//
Integer IncrSequence::squashedIndex() const
{
    int index;
    Integer result=0;
    for (index=0; index<_count; index++) {
	add(result,Choose(_arr[index],index+1),result);
    }
    return result;
}

//
// This constructor creates an n-set whose "squashedIndex" equals
// the passed Integer, "squashed"
//
IncrSequence::IncrSequence(int n, Integer squashed)
	: _count(n), _arr(new int[n])
{
    int ind;
    for (ind=n-1; ind>=0; ind--) {
	int value=ind+1;
	while (squashed>=Choose(value,ind+1)) {
	    value++;
        }
	_arr[ind]=value-1;
	squashed -= Choose(value-1,ind+1);
    }
}

ostream &IncrSequence::write(ostream &out) const
{
    int i;
    out << "[ ";
    for (i=0; i<_count; i++ ) {
	out << _arr[i] << (((i&15)==15) ? "\n" : " ");
    }
    return (out << "]");
}

